﻿
' Interface to the Robot

Imports System.Net
Imports System.Net.Sockets
Imports System.Text


Module modBotIF


    Public Function ftnBot_Transmit(ByVal strCommand As String, ByVal strResponse As String, ByVal intTimeOut As Integer) As String
        If boolComPort Then
            ftnBot_Transmit = ftnBot_ComPortTransmit(strCommand, strResponse, intTimeOut)
        Else
            ftnBot_Transmit = ftnBot_SocketTransmit(strCommand, strResponse, intTimeOut)
        End If
    End Function






    '============================================  SOCKET IF  ===============================================
    Public Bot_Socket As Socket = New Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp)



    ' -----------------------------------------------------------------------------------
    ' The socket communication code (was originally) based upon a demo by:
    ' Rimmel© nemo1966:gmail.com 
    ' It has since been changed beyond recognition...
    ' -----------------------------------------------------------------------------------
    ' Orininal Note (may no longer apply) : An error can occur if the recv byte length on the last call is exactly 
    ' 256 bytes. Will  cause the receive call in the loop to block (waiting).
    ' This is where an asynchronous solution would be better.
    ' -----------------------------------------------------------------------------------


    Public Function ftnBot_SocketTransmit(ByVal strCommand As String, ByVal strResponse As String, ByVal intTimeOut As Integer) As String
        ftnBot_SocketTransmit = ""
        Dim SendBytes As [Byte]() = Encoding.ASCII.GetBytes(strCommand & vbCrLf) ' Convert the ASCII command into bytes, adding a line termination on (vbCrLf)
        System.Windows.Forms.Application.DoEvents()
        If (Bot_Socket Is Nothing) Or (Not Bot_Socket.Connected) Then BotConnect()
        If (Bot_Socket.Connected) Then
            Try
                Bot_Socket.Send(SendBytes, SendBytes.Length, SocketFlags.None)
                If (intTimeOut <> 0) Then ftnBot_SocketTransmit = ftnWaitForExpectedResponse(strResponse, intTimeOut, strCommand)
            Catch oEX As Exception
                MsgBox(Color.Red, " ERROR: Socket Exception While Connecting: " & vbCrLf & " Error Message Received: " & oEX.Message & vbCrLf & "Please restart GotATranslator")
                GotATranslatorClass.WriteComms(Color.Red, " SocketException While Connecting: " & ".    Error Message Received: " & oEX.Message)
                GotATranslatorClass.WriteComms(Color.Red, System.Environment.StackTrace)
                System.Windows.Forms.Application.DoEvents()
                GotATranslatorClass.subWait(1000)
                BotDisconnect()
            End Try
        End If
    End Function


    Private Sub BotConnect()
        Dim ep As IPEndPoint

        ep = New IPEndPoint(Robot_IPAddress, Robot_Port)  ' Create an IPEndpoint (ep)
        System.Windows.Forms.Application.DoEvents()
        Try ' Connect
            Bot_Socket.Blocking = True
            Bot_Socket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, True)
            Bot_Socket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.SendTimeout, 1000)
            Bot_Socket.ReceiveTimeout = 1000
            Bot_Socket.SendTimeout = 1000
            Bot_Socket.Connect(ep)
            System.Windows.Forms.Application.DoEvents()

            If Not Bot_Socket.Connected Then
                System.Windows.Forms.Application.DoEvents()
            Else
                Bot_Socket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.SendTimeout, 1000)
                Bot_Socket.ReceiveTimeout = 1000
                GotATranslatorClass.subWait(100)
            End If
        Catch oEX As SocketException
            GotATranslatorClass.WriteComms(Color.Red, " SocketException While Connecting.   " & " Error Message Received: " & oEX.Message)
            GotATranslatorClass.WriteComms(Color.Red, System.Environment.StackTrace)
            System.Windows.Forms.Application.DoEvents()
            BotDisconnect()
        End Try
        ep = Nothing
    End Sub


    Public Sub BotDisconnect()
        Dim boolReuseSocket As Boolean = True

        Try ' DisConnect
            If Bot_Socket IsNot Nothing Then
                Bot_Socket.Disconnect(boolReuseSocket)
                Bot_Socket = New Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp)
            End If
        Catch oEX As SocketException
            GotATranslatorClass.WriteComms(Color.Red, " SocketException on Disconnect.   " & " Message Received: " & oEX.Message)
            GotATranslatorClass.WriteComms(Color.Red, System.Environment.StackTrace)
        End Try
        GotATranslatorClass.Refresh()
    End Sub



    Private Function ftnWaitForExpectedResponse(ByVal strTextToWaitFor As String, ByVal intTimeOut As Integer, ByVal strCommand As String) As String
        Dim Time_Out, TimeNow As DateTime
        ftnWaitForExpectedResponse = ""
        On Error Resume Next
        System.Windows.Forms.Application.DoEvents()
        TimeNow = Now()
        Time_Out = TimeNow.AddMilliseconds(intTimeOut)
        If Bot_Socket.Connected Then
            Do
                ftnWaitForExpectedResponse = ftnWaitForExpectedResponse & ftnGetBotResponse(100)
                System.Windows.Forms.Application.DoEvents()
                TimeNow = Now()
            Loop Until ((TimeNow > Time_Out) Or CBool(InStr(ftnWaitForExpectedResponse, strTextToWaitFor)))
            If (TimeNow > Time_Out) Then GotATranslatorClass.WriteComms(Color.Orange, Now() & " }}}}}}}}}}  WARNING: Bot Communication Timeout while waiting for " & strTextToWaitFor)
        Else
            ftnWaitForExpectedResponse = ""
            GotATranslatorClass.tsslStatus.Text = ":  WARNING - Bot not connected"
            System.Windows.Forms.Application.DoEvents()
        End If
    End Function







    Public Function ftnGetBotResponse(ByVal intTimeOut As Integer) As String
        Dim Time_Out As DateTime
        Dim NumBytes As Integer = 0 ' NumBytes: Number of bytes returned from telnet socket (count)
        Dim RecvBytes(255) As [Byte] ' Create a byte array for receiving bytes from the telnet socket
        ftnGetBotResponse = ""
        Dim timeNow As DateTime
        ftnGetBotResponse = ""
        System.Windows.Forms.Application.DoEvents()
        Time_Out = Now.AddMilliseconds(intTimeOut)
        If Bot_Socket.Connected Then
            Do
                Try
                    If (Bot_Socket.Poll(1000000, SelectMode.SelectRead)) Then _
                          NumBytes = Bot_Socket.Receive(RecvBytes, RecvBytes.Length, 0) ' numbytes =count of bytes returned
                    ftnGetBotResponse = ftnGetBotResponse + Encoding.ASCII.GetString(RecvBytes, 0, NumBytes)
                Catch ex As Exception
                    ftnGetBotResponse = ""
                    BotDisconnect()
                End Try
                System.Windows.Forms.Application.DoEvents()
                timeNow = Now()
            Loop Until ((timeNow > Time_Out) Or CBool(InStr(Right$(ftnGetBotResponse, 2), vbCrLf)))
        Else
            ftnGetBotResponse = ""
            GotATranslatorClass.tsslStatus.Text = ":  WARNING - Bot not connected"
            System.Windows.Forms.Application.DoEvents()
        End If

    End Function




    '========================================  ComPort IF  =============================================================
  
    Public ComPort As New System.IO.Ports.SerialPort   'declares a handle for COM port 


    Public Function ftnBot_ComPortTransmit(ByVal strCommand As String, ByVal strResponse As String, ByVal intTimeOut As Integer) As String
        ftnBot_ComPortTransmit = ""
        If (ComPort Is Nothing) Or (Not ComPort.IsOpen) Then ftnOpenComPort()
        If (ComPort.IsOpen) Then
            Try
                ComPort.WriteLine(strCommand)
                If (intTimeOut <> 0) Then
                    Dim Time_Out As DateTime = Now().AddMilliseconds(intTimeOut)
                    If ComPort.IsOpen Then
                        Do
                            If (ComPort.BytesToRead > 0) Then ftnBot_ComPortTransmit = ftnBot_ComPortTransmit & (ComPort.ReadExisting)
                            System.Windows.Forms.Application.DoEvents()
                        Loop Until ((Now() > Time_Out) Or CBool(InStr(ftnBot_ComPortTransmit, strResponse)))
                        If (Now() > Time_Out) Then GotATranslatorClass.WriteComms(Color.Orange, Now() & " }}}}}}}}}}  WARNING: Bot Communication Timeout while waiting for " & strResponse)
                        ComPort.DiscardInBuffer()
                        ComPort.DiscardOutBuffer()
                    End If
                End If
            Catch oEX As Exception
                GotATranslatorClass.WriteComms(Color.Red, " ERROR: ComPort: " & ".    Error Message Received: " & oEX.Message)
                GotATranslatorClass.WriteComms(Color.Red, System.Environment.StackTrace)
                System.Windows.Forms.Application.DoEvents()
                CloseComPort()
            End Try
        End If

    End Function



    Private Function ftnOpenComPort() As Boolean
        ftnOpenComPort = False
        Try
            If (ComPort.IsOpen) Then  ComPort.Close() 'if program has previously opened a COM port, start by closing it
            ComPort = My.Computer.Ports.OpenSerialPort(strComPort, 19200, 0, 8, 2) ' attempt to open a new COM port
            ComPort.DiscardInBuffer()
            ComPort.DiscardOutBuffer()
            ComPort.ReadTimeout = 2000                 'we will only arrive here on successfull open
            ComPort.WriteTimeout = 2000
            ftnOpenComPort = ComPort.IsOpen
        Catch                                       'catch open failures
            CloseComPort()
            ftnOpenComPort = False
        End Try
        If (Not ftnOpenComPort And ComPort.IsOpen) Then CloseComPort()
    End Function


    Private Sub CloseComPort()
        If ComPort.IsOpen Then
            ComPort.DiscardInBuffer()
            ComPort.DiscardOutBuffer()
            ComPort.Close()
        End If
        ComPort.Dispose()
    End Sub




End Module
